/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: hmr2300.c,v 1.17 2008/02/16 15:28:20 phk Exp $
 *
 */

#include <stdio.h>
#include <pthread.h>
#include <configkit.h>
#include <termios.h>
#include <assert.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "measured.h"
#include "configkit_conf.h"

#define HMR2300_NPOINTS	3

struct hmr2300 {
	unsigned		magic;
#define HMR2300_MAGIC 		0x83742834
	struct ck_instance	cki;
	struct group		*gp;
	pthread_mutex_t		lock;

	unsigned		debug;
	double			pollrate;
	struct serial		*serial;
	struct point		*p[HMR2300_NPOINTS];
	double			o[HMR2300_NPOINTS];
};

static void *
hmr2300_thread(void *priv)
{
	struct termios t;
	struct hmr2300 *sc;
	char buf[128], *p, *q;
	int i;
	int x[HMR2300_NPOINTS];

	CAST_OBJ_NOTNULL(sc, priv, HMR2300_MAGIC);

	AZ(tcgetattr(serial_fd(sc->serial), &t));
	cfmakeraw(&t);
	t.c_cflag |= CLOCAL;
	AZ(tcsetattr(serial_fd(sc->serial), TCSAFLUSH, &t));
	
	for (i = 0; i < 3; i++) {
		serial_write(sc->serial, "\033");
		sleep(1);
	}
	while (1) {
		i = serial_read(sc->serial, 
		    buf, 28, sizeof buf, sc->pollrate * 1e3);
		assert(i >= 0);
		if (i == 0) {
			serial_write(sc->serial, "*99P\r");
			continue;
		}
		if (i > 0)
			buf[i] = '\0';
		if (sc->debug & 2)
			SCDebug(sc, "read %d %V", i, buf);

		/* Edit the string */
		i = 0;
		for (q = p = buf; *p; p++) {
			if (*p == ',')
				continue;
			if (i && *p == ' ')
				continue;
			if (*p == '-' || *p == ' ')
				i = 1;
			else
				i = 0;
			*q++ = *p;
		}
		*q = '\0';
		if (3 != sscanf(buf, "%d %d %d", &x[0], &x[1], &x[2])) {
			SCDebug(sc, "HMR2300: %d %V", buf);
			serial_write(sc->serial, "\033");
			sleep(1);
			AZ(tcsetattr(serial_fd(sc->serial),
			    TCSAFLUSH, &t));
			continue;
		}
		for (i = 0; i < 3; i++)
			SetVal(sc->p[i], (double)x[i] / 15000.0 - sc->o[i]);
	}
}

static void
hmr2300_init(CK_ARG, struct hmr2300 *sc)
{
	unsigned u;
	const char *l = "X\0Y\0Z\0";

	sc->pollrate = 10.0;
	serial_init(CK_PASS, &sc->serial, hmr2300_thread, sc);
	for (u = 0; u < HMR2300_NPOINTS; u++) {
		sc->p[u] = CreatePoint(sc->gp, u, AIN);
		SetPointLabel(sc->p[u], l + u + u);
		SetPointUnit(sc->p[u], "Gauss");
		SetPointFormat(sc->p[u], "%.6f");
	}
}

static void
hmr2300_deinit(CK_ARG, struct hmr2300 *sc)
{

	serial_destroy(CK_PASS, &sc->serial);
}

SerialFunc(hmr2300, HMR2300_MAGIC, serial);
RealVar(hmr2300, HMR2300_MAGIC, x0, o[0], 0.0, "%.6f", );
RealVar(hmr2300, HMR2300_MAGIC, y0, o[1], 0.0, "%.6f", );
RealVar(hmr2300, HMR2300_MAGIC, z0, o[2], 0.0, "%.6f", );
UintVar(hmr2300, HMR2300_MAGIC, debug, debug, 0, );
RealVar(hmr2300, HMR2300_MAGIC, pollrate, pollrate, 10.0, "%.3f", );
MODULE_GROUP(hmr2300, HMR2300_MAGIC);
