/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: serspy.c,v 1.3 2008/02/17 09:55:54 phk Exp $
 *
 * Receive and decode the serial SAVS2000->VindPanel data stream.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <pthread.h>
#include <stdarg.h>
#include <configkit.h>
#include <termios.h>
#include <errno.h>
#include <math.h>
#include <ctype.h>
#include <unistd.h>

#include "measured.h"
#include "configkit_conf.h"

/* Softc -------------------------------------------------------------*/

struct serspy {
	unsigned		magic;
#define SERSPY_MAGIC		0x3eac923c
	struct group		*gp;
	struct ck_instance	cki;
	pthread_mutex_t		lock;

	double			nchar;
	unsigned		debug;
	struct serial		*serial;
	struct point		*pt;
};

/* Serial port listening ---------------------------------------------*/

#define BUFLEN		100
static void *
serspy_thread(void *foo)
{
	struct serspy *sc;
	char buf[BUFLEN + 1];
	int i;

	CAST_OBJ_NOTNULL(sc, foo, SERSPY_MAGIC);

	while (1) {
		i = serial_read(sc->serial, buf, 1, BUFLEN, 10000);
		if (i == 0)
			continue;
		assert(i > 0);
		buf[i] = '\0';
		if (sc->debug)
			SCDebug(sc, "<%V>", buf);
		Record(GI(sc->gp), i, buf);
		sc->nchar += i;
		SetVal(sc->pt, sc->nchar);
	}
}

/* ConfigKit routines ------------------------------------------------*/

static void
serspy_init(CK_ARG, struct serspy *sc)
{

	serial_init(CK_PASS, &sc->serial, serspy_thread, sc);
	sc->pt = CreatePoint(sc->gp, 1, COUNT);
}

static void
serspy_deinit(CK_ARG, struct serspy *sc)
{

	serial_destroy(CK_PASS, &sc->serial);
}

SerialFunc(serspy, SERSPY_MAGIC, serial);
UintVar(serspy, SERSPY_MAGIC, debug, debug, 0, );
MODULE_GROUP(serspy, SERSPY_MAGIC);
