/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: test.c,v 1.2 2008/02/16 16:13:16 phk Exp $
 *
 */

#include <stdio.h>
#include <pthread.h>
#include <configkit.h>
#include <assert.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "measured.h"
#include "configkit_conf.h"

#define TEST_NPOINTS	5

struct test {
	unsigned		magic;
#define TEST_MAGIC 		0x3ad5d1d2
	struct ck_instance	cki;
	struct group		*gp;
	pthread_t		tp;
	pthread_mutex_t		lock;

	struct point		*p[TEST_NPOINTS];
	double			v[TEST_NPOINTS];
};

static void *
test_thread(void *priv)
{
	struct test *sc;
	int i;

	CAST_OBJ_NOTNULL(sc, priv, TEST_MAGIC);

	while (1) {
		AZ(pthread_mutex_lock(&sc->lock));
		sc->v[0] = sc->v[1] * 355/113;
		sc->v[2] = !sc->v[3];
		sc->v[4]++;
		for (i = 0; i < TEST_NPOINTS; i++)
			SetVal(sc->p[i], sc->v[i]);
		AZ(pthread_mutex_unlock(&sc->lock));
		sleep(1);
	}
}

static void
test_control(struct point *p, void *p1, void *p2, double val)
{
	struct test *sc;
	double *vp;

	CAST_OBJ_NOTNULL(sc, p1, TEST_MAGIC);
	vp = p2;
	
	AZ(pthread_mutex_lock(&sc->lock));
	*vp = val;
	SetVal(p, *vp);
	AZ(pthread_mutex_unlock(&sc->lock));
	PDebug(p, "TestControl(%g)", val);
}

static void
test_init(CK_ARG, struct test *sc)
{

	sc->p[0] = CreatePoint(sc->gp, 0, AIN);
	sc->p[1] = CreatePoint(sc->gp, 1, AOUT);
	sc->p[2] = CreatePoint(sc->gp, 2, DIN);
	sc->p[3] = CreatePoint(sc->gp, 3, DOUT);
	sc->p[4] = CreatePoint(sc->gp, 4, COUNT);
	SetPointControl(sc->p[1], test_control, sc, &sc->v[1]);
	SetPointControl(sc->p[3], test_control, sc, &sc->v[3]);
	AZ(pthread_create(&sc->tp, NULL, test_thread, sc));
}

static void
test_deinit(CK_ARG, struct test *sc)
{
	void *vp;

	AZ(pthread_mutex_lock(&sc->lock));
	AZ(pthread_cancel(sc->tp));
	AZ(pthread_join(sc->tp, &vp));
	AZ(pthread_mutex_unlock(&sc->lock));
}

MODULE_GROUP(test, TEST_MAGIC);
