/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: vor4000_proto.h,v 1.4 2006/01/18 21:49:58 phk Exp $
 *
 * Protocol field definitions for the RCMS 4000 protocol.
 *
 * Take from a photocopy of chapter 5 "REMOTE CONTROL AND MONITORING SYSTEM
 * 4000 (RCMS)" marked "Ed. 6.91" from an unidentified manual.
 *
 */

#define STATION_MASK			0x07
#define TYPE_MASK			0x18U
#define TYPE_SHIFT			0x3U
#define MSG_MAXLEN			11

#define C0_LEN				0x03 /* Type 0 control message */

#define C0B0_TYPE			0x00

#define C0B1_SWITCH_CONTROL		0x01 /* 1 = on; 0 = off */
#define C0B1_TX1_ON_MAIN		0x02
#define C0B1_TX2_ON_MAIN		0x04
#define C0B1_TX1_TX2_OFF		0x08

#define C1_LEN				0x03 /* Type 1 control message */

#define C1B0_TYPE			0x08
#define C1B0_DIFF_MON			0x20

#define C1B0_SWITCH_CONTROL		0x01 /* 1 = on; 0 = off */
#define C1B1_TX1_ON_MAIN		0x02
#define C1B1_TX2_ON_MAIN		0x04
#define C1B1_TX1_TX2_OFF		0x08

#define C2_LEN				0x03 /* Type 2 control message */

#define C2B0_TYPE			0x10
#define C2B0_DIFF_READ			0x20
#define C2B0_DIFF_MON			0x40

/* Type 3 control message */
#define C3_LEN				0x05

#define C3B0_TYPE			0x18
#define C3B0_DIFF_MON			0x20

#define S0_LEN				0x04 /* Type 0 signalling message */

#define S0B0_TYPE			0x00
#define S0B0_DVOR			0x20
#define S0B0_100WTX			0x40
#define S0B0_BATT_RESID_CHARGE		0x80

#define S0B1_ALARM			0x01 /* (system off) */
#define S0B1_SHUT_MAIN_TX		0x02 /* (Standby TX on) */
#define S0B1_BITE_ALARM			0x04 /* (Prealarm || Temp Alarm) */
#define S0B1_MON_BYPASS_OR		0x08 /* (or gate) */
#define S0B1_MON_FAULT_OR		0x10 /* (or gate) */
#define S0B1_TX1_MAIN			0x20
#define S0B1_MAINS_OFF			0x40
#define S0B1_WARNING			0x80

#define S1_LEN				0x0b /* Type 1 signalling message */

#define S1B0_TYPE			0x08
#define S1B0_DIFF_MON			0x20
#define S1B0_REMOTE			0x80 /* disable = 0, enable = 1 */

#define S1B1_BITE_WARNING		0x02
#define S1B1_MONITOR_WARNING		0x04
#define S1B1_MONITOR_FAULT		0x08
#define S1B1_COMONITOR_FAULT		0x10
#define S1B1_OPERATION_OFF		0x80

#define S1B2_OPER_ALARM_2ND_ORDER	0x04
#define S1B2_TX_ON_ANTENNA		0x08
#define S1B2_OPER_ALARM_1ST_ORDER	0x40
#define S1B2_MODULATOR_ON		0x80

#define S1B3_AZIMUTH			0x01
#define S1B3_30HZ_MOD			0x02
#define S1B3_9960HZ_MOD			0x04
#define S1B3_FREQ			0x08
#define S1B3_IDENT			0x10
#define S1B3_RF_LEVEL			0x20
#define S1B3_30HZ_FM			0x40
#define S1B3_RF_PHASE			0x80

#define S1B4_5V_TX1			0x01 /* (neg.logic) 0 = TX1 on */
#define S1B4_5V_TX2			0x02 /* (neg.logic) 0 = TX2 on */
#define S1B4_BCPS_FAULT			0x04
#define S1B4_MON_BYPASS			0x08
#define S1B4_ACK_EGB			0x10
#define S1B4_REM_EN_DIS			0x20 /* 0 = local, 1 = remote */
#define S1B4_MON_ALARM			0x80

#define S1B5_ALARM_SYS_OFF		0x01 /* (system off) */
#define S1B5_SHUT_MAIN_TX		0x02 /* (Standby TX on) */
#define S1B5_BITE_ALARM			0x04 /* (Prealarm || Temp Alarm) */
#define S1B5_MON_BYPASS_OR		0x08 /* (or gate) */
#define S1B5_MON_FAULT			0x10 /* (or gate) */
#define S1B5_TX1_MAIN			0x20
#define S1B5_MAINS_OFF			0x40
#define S1B5_WARNING			0x80

#define S2_LEN				0x04 /* Type 2 signalling message */

#define S2B0_TYPE			0x10

#define PARAM_AZIMUTH			0x20
#define SCALE_AZIMUTH			(360.0/65536.0)
#define PARAM_RF_LEVEL			0x21
#define SCALE_RF_LEVEL			(100.0/16384.0)
#define PARAM_30HZ_MOD			0x23
#define SCALE_30HZ_MOD			(100.0/32768.0)
#define PARAM_9960HZ_MOD		0x24
#define SCALE_9960HZ_MOD		(100.0/32768.0)
#define PARAM_30HZ_FM_DEV		0x25
#define SCALE_30HZ_FM_DEV		(100.0/32768.0)
