#include <stdio.h>
#include <fcntl.h>
#include <err.h>
#include <termios.h>
#include <assert.h>
#include <sys/time.h>


static void
openserial(const char *name, int *fd)
{
	struct termios t;
	int i;

	*fd = open(name, O_RDWR);
	if (*fd < 0)
		err(1, "Cannot open %s", name);
	
        i = tcgetattr(*fd, &t);
        if (i < 0) {
                close(*fd);
                *fd = -1;
                err(1, "tcgetattr failed on %s\n", name);
        }
 
        cfmakeraw(&t);
        cfsetspeed(&t, B600);
        t.c_cc[VMIN] = 1;
        t.c_cc[VTIME] = 0;
        t.c_cflag |= CLOCAL;
        t.c_cflag &= ~CRTSCTS;
        t.c_cflag &= ~MDMBUF;
        t.c_cflag &= ~HUPCL;
        t.c_cflag |= CREAD;
        t.c_cflag |= CS8;
        t.c_cflag |= CSTOPB;
        t.c_cflag |= PARENB;
        t.c_cflag |= CDSR_OFLOW;
                 
        i = tcsetattr(*fd, TCSANOW, &t);
        if (i < 0) {
                close(*fd);
                *fd = -1;
                err(1, "tcsetattr failed on %s\n", name);
        }
}

double
delta_t(const struct timespec *t1, const struct timespec *t2)
{
        double d;

        d = t1->tv_sec - t2->tv_sec;
        d += 1e-9 * (t1->tv_nsec - t2->tv_nsec);
        return (d);
}

int
main(int argc, char **argv)
{
	FILE *fi;
	char buf[BUFSIZ];
	char *s;
	int i, j, k, jl;
	struct timespec ts, now;
	int fd1, fd2;
	u_char c;
	double dt, when;

	assert(argc == 2);
	fi = fopen(argv[1], "r");
	if (fi == NULL)
		err(1, "cannot open %s", argv[1]);

	openserial("/dev/nmdm0A", &fd1);
	openserial("/dev/nmdm1A", &fd2);

	jl = 0;
	dt = 0;
	while (fgets(buf, sizeof buf, fi)) {
		s = strchr(buf, '\n');
                if (s != NULL)
                        *s = '\0';
                i = sscanf(buf, "%d.%ld chr %d %d",
                    &ts.tv_sec, &ts.tv_nsec, &j, &k);
                if (i != 4) {
                        if (0)
                                printf("i %d j %d k %d <%s>\n", i, j, k, buf);
                        continue;
                }
		if (dt == 0) {
			clock_gettime(CLOCK_MONOTONIC, &now);
			dt = delta_t(&ts, &now);
		}
		if (jl != j) {
			if (j == 5)
				fprintf(stderr, "\n");
			else
				fprintf(stderr, " ");
			clock_gettime(CLOCK_MONOTONIC, &now);
			when = delta_t(&ts, &now) - dt;
			if (when > 10.0) {
printf("\nWhen %f\n", when);
				sleep(1);
				clock_gettime(CLOCK_MONOTONIC, &now);
				dt = delta_t(&ts, &now);
			} else if (when > 0.0) {
				when /= 10;
				usleep((int)(when * 1e6));
			}
		}
		c = k;
		if (j == 3) {
			if (isatty(2))
				fprintf(stderr, "\033[1m%02x\033[m", k);
			else
				fprintf(stderr, "%02x", k);
			i = write(fd1, &c, 1);
			assert(i == 1);
		} else if (j == 5) {
			fprintf(stderr, "%02x", k);
			i = write(fd2, &c, 1);
			assert(i == 1);
		} else {
			fprintf(stderr, "%c", k);
		}
		jl = j;
	}

	return (0);
}
