/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: httpd_pattern.c,v 1.10 2007/03/27 12:09:53 phk Exp $
 *
 */

#include <assert.h>
#include <pthread.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>

#include "measured.h"
#include "measured_int.h"

static const char *bgcol[] = {
	[L_LOW] =	" style=\"background: #8080ff\"",
	[L_SAG] =	" style=\"background: #c0c0ff\"",
	[L_RAISE] =	" style=\"background: #ffc0c0\"",
	[L_HIGH] =	" style=\"background: #ff8080\"",
	[L_NORMAL] =	" style=\"background: #ffffff\"",
	[L_TIMEOUT] =	" style=\"background: #000000\""
};

static void
site_html(FILE *f, const struct site *sp, const struct site *lsp)
{

	if (lsp != NULL)
		CHECK_OBJ(lsp, SITE_MAGIC);
	if (sp != NULL) {
		CHECK_OBJ(sp, SITE_MAGIC);
		fprintf(f, "<HR>\n");
		fprintf(f, "<H1>site <A href=\"/data?%u\">",
		    SITE_OF(sp->instance));
		fprintf(f, "%03u</A></H1>\n", SITE_OF(sp->instance));
	}
}

static void
group_html(FILE *f, const struct group *gp, const struct group *lgp)
{

	if (lgp != NULL) {
		CHECK_OBJ(lgp, GROUP_MAGIC);
		fprintf(f, "</TABLE>\n");
	}
			
	if (gp != NULL) {
		CHECK_OBJ(gp, GROUP_MAGIC);
		fprintf(f, "<HR>\n");
		fprintf(f, "<H2>group <A href=\"/data?%u.%u\">",
		    SITE_OF(gp->instance), GROUP_OF(gp->instance));
		fprintf(f, "%03u.%03u</A> %s</H2>\n",
		    SITE_OF(gp->instance),
		    GROUP_OF(gp->instance), gp->name);
		fprintf(f, "<TABLE cellpadding=\"2\" cellspacing=\"2\"");
		fprintf(f, " border=\"1\">\n");
		fprintf(f, "<TR>\n");
		fprintf(f, "<TH>Index</TH>");
		fprintf(f, "<TH>Type</TH>\n");
		fprintf(f, "<TH>Label</TH>\n");
		fprintf(f, "<TH>Value</TH>\n");
		fprintf(f, "<TH>Action</TH>\n");
		fprintf(f, "<TH>Units</TH>\n");
		fprintf(f, "<TH%s>Low</TH>\n", bgcol[L_LOW]);
		fprintf(f, "<TH%s>Sag</TH>\n", bgcol[L_SAG]);
		fprintf(f, "<TH%s>Raise</TH>\n", bgcol[L_RAISE]);
		fprintf(f, "<TH%s>High</TH>\n", bgcol[L_HIGH]);
		fprintf(f, "<TH>Trap</TH>\n");
		fprintf(f, "</TR>\n");
	}
}

static void
point_html(FILE *f, const struct point *pp)
{

	fprintf(f, "<TR>\n");
	fprintf(f, "<TD>%P</TD>", pp);
	fprintf(f, "<TD>%s</TD>", p_Type(pp));
	fprintf(f, "<TD>");
	fprintf(f, "%s", p_label(pp));
	fprintf(f, "</TD>");
	fprintf(f, "<TD%s>", bgcol[pp->curlim]);
	fprintf(f, p_format(pp), pp->value);
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
	if (p_type(pp) == DOUT) {
		fprintf(f, "<form METHOD=GET");
		fprintf(f, " ACTION=\"point_set\"><p style=\"padding: 0 0 0 0; margin: 0 0 0 0;\">\n");
		fprintf(f, "<input TYPE=HIDDEN NAME=point VALUE=%P>\n", pp);
		fprintf(f, "<input TYPE=SUBMIT NAME=val VALUE=%d>",
		    pp->value > .5 ? 0 : 1);
		fprintf(f, "</p></form>\n");
	} else if (p_type(pp) == AOUT) {
		fprintf(f, "<form METHOD=GET");
		fprintf(f, " ACTION=\"point_set\"><p style=\"padding: 0 0 0 0; margin: 0 0 0 0;\">\n");
		fprintf(f, "<input TYPE=HIDDEN NAME=point VALUE=%P>\n", pp);
		fprintf(f, "<input TYPE=TEXT NAME=val SIZE=8 VALUE=");
		fprintf(f, p_format(pp), pp->value );
		fprintf(f, "></p></form>");
	}
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
	fprintf(f, "%s", p_unit(pp));
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
/*lint -save -e777 */
	if (!isnan(pp->limits[L_LOW]))
		fprintf(f, p_format(pp), pp->limits[L_LOW]);
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
	if (!isnan(pp->limits[L_SAG]))
		fprintf(f, p_format(pp), pp->limits[L_SAG]);
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
	if (!isnan(pp->limits[L_RAISE]))
		fprintf(f, p_format(pp), pp->limits[L_RAISE]);
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
	if (!isnan(pp->limits[L_HIGH]))
		fprintf(f, p_format(pp), pp->limits[L_HIGH]);
/*lint -restore */
	fprintf(f, "</TD>");
	fprintf(f, "<TD>");
	if (pp->trap != 0)
		fprintf(f, "%u", pp->trap);
	fprintf(f, "</TD>");
	fprintf(f, "</TR>\n");
}

static void
url_data(URL_ARGS)
{
	struct site *sp, *lsp = NULL;
	struct group *gp, *lgp = NULL;
	struct point *pp;
	struct pattern *pt;

	if (h->args == NULL)
		pt = CompilePattern("*");
	else
		pt = CompilePattern(h->args);

	Httpd200(f);
	HtmlHead(f, "Measured");
	TopoHold();
	TAILQ_FOREACH(sp, &all_sites, list) {
		TAILQ_FOREACH(gp, &sp->groups, list) {
			TAILQ_FOREACH(pp, &gp->points, list) {
				if (!MatchPattern(pt, pp->instance))
					continue;
				if (lsp != sp) {
					group_html(f, NULL, lgp);
					lgp = NULL;
					site_html(f, sp, lsp);
				}
				lsp = sp;
				if (lgp != gp)
					group_html(f, gp, lgp);
				lgp = gp;
				point_html(f, pp);
			}
		}
	}
	group_html(f, NULL, lgp);
	site_html(f, NULL, lsp);
	HtmlTail(f);
	TopoRel();
	FreePattern(pt);
}

static void
url_root(URL_ARGS)
{

	fprintf(f, "HTTP/1.1 302 Found\r\n");
	fprintf(f, "Location: /data\r\n");
	fprintf(f, "\r\n");
}

void
InitUrlPattern(void)
{

	RegisterUrl("GET", "/data", url_data, NULL);
	RegisterUrl("GET", "/", url_root, NULL);
}
