/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: measured.h,v 1.110 2008/04/29 19:45:14 phk Exp $
 *
 */

#include "phk_obj.h"
#include "measured_api.h"

struct site;
struct group;
struct point;
struct pattern;

/*lint -e726*/
typedef enum event_e {
#define EVD(a, b, ...)	E_##a = b,
#include "events.h"
#undef EVD
} event_t;
/*lint +e726*/

/* Module related */
void module_ctor(void *np);

#define MODULE_GROUP(nm,NM)					\
	GROUP(nm, NM);						\
	void nm##_ctor(void);					\
	void							\
	nm##_ctor(void)						\
	{							\
		module_ctor(&cfg_##nm);				\
	}							\
	asm(".section .ctors,\"aw\",@progbits\n.long " #nm "_ctor\n");

/* Sites */
struct site *CreateSite(unsigned instance);
void CleanSite(const struct site *);
void DestroySite(struct site *);

/* Groups */
struct group *CreateGroup(struct site *s, const char *name, unsigned instance);
void DestroyGroup(struct group *);
void CleanGroup(const struct group *gp);

/* Points */

typedef void control_t(struct point *, void *priv1, void *priv2, double val);

struct point *CreatePoint(struct group *gp, unsigned idx, enum point_type type);
void DestroyPoint(struct point *);
void SetVal(struct point *pp, double v);
void SetValBool(struct point *pp, int v);
double p_value(const struct point *pp);
enum point_type p_type(const struct point *pp);
const char *p_Type(const struct point *pp);

void SetPointControl(struct point *pp, control_t *func, void *priv1, void *priv2);

void SetPointLabel(struct point *pp, const char *label);
const char *p_label(const struct point *pp);
void SetPointUnit(struct point *pp, const char *unit);
const char *p_unit(const struct point *pp);
void SetPointFormat(struct point *pp, const char *format);
const char *p_format(const struct point *pp);
void SetPointTimeout(struct point *pp, double timeout);
double p_timeout(const struct point *pp);

struct point *FindPoint(unsigned instance);
const char *PointFmt(const struct point *pp, double v, char *buf, unsigned buflen);

unsigned SI(const struct site *);
unsigned GI(const struct group *);
unsigned PI(const struct point *);

void ControlPointI(unsigned instance, double val);

struct ptdesc {
	enum point_type	type;
	const char	*label;
	unsigned	idx;
	unsigned	point;
};

unsigned CreatePoints(struct group *gp, struct point ***pp, struct ptdesc *pt);

#ifdef CK_ARG
int ConfigPoint(CK_ARG, struct group *gp, const char **point, char **argv);
#endif

/* Pattern matches */
#include "pattern.h"

/* Logging and reporting */
extern int d_flag;
void Debug(unsigned instance, const char *fmt, ...);
#define PDebug(pt, ...) Debug(PI(pt), __VA_ARGS__)
#define GDebug(gp, ...) Debug(GI(gp), __VA_ARGS__)
#define SCDebug(sc, ...) Debug(GI(sc->gp), __VA_ARGS__)
#define NSCDebug(sc, n, ...) do {				\
	if (sc->debug & n) Debug(GI(sc->gp), __VA_ARGS__);	\
	} while (0)
void Notice(unsigned instance, const char *fmt, ...);
#define PNotice(pt, ...) Notice(PI(pt), __VA_ARGS__)
#define GNotice(gp, ...) Notice(GI(gp), __VA_ARGS__)
#define SCNotice(sc, ...) Notice(GI(sc->gp), __VA_ARGS__)
void Log(unsigned instance, const char *fmt, ...);
#define PLog(pt, ...) Log(PI(pt), __VA_ARGS__)
#define GLog(gp, ...) Log(GI(gp), __VA_ARGS__)
#define SCLog(sc, ...) Log(GI(sc->gp), __VA_ARGS__)
void Error(unsigned instance, const char *fmt, ...);
#define PError(pt, ...) Error(PI(pt), __VA_ARGS__)
#define GError(gp, ...) Error(GI(gp), __VA_ARGS__)
#define SCError(sc, ...) Error(GI(sc->gp), __VA_ARGS__)
void __Fatal(const char *func, const char *file, int line, const char *fmt, ...) __dead2;
/* void __assert(const char *func, const char *file, int line, const char *string) __dead2; */
#define Fatal(fmt, ...) \
	__Fatal(__func__, __FILE__, __LINE__, fmt, __VA_ARGS__)

void Record(unsigned point, unsigned len, const void *ptr);

/* Httpd -------------------------------------------------------------*/

#define MAX_HTTP_PARTS	32

struct http {
	char *req;
	char *uri;
	char *url;
	char *proto;
	char *headers[MAX_HTTP_PARTS];
	char *args;
	char *argv[MAX_HTTP_PARTS];
};

#define URL_ARGS FILE *f __unused, struct http *h __unused, void *priv __unused
typedef void url_f(URL_ARGS);
void RegisterUrl(const char *verb, const char *url, url_f *func, void *priv);
char *HttpHeader(const struct http *h, const char *hdr);
char *HttpArg(const struct http *h, const char *hdr);
void Httpd200(FILE *f);
void HtmlHead(FILE *f, const char *title);
void HtmlTail(FILE *f);
int Httpd_Auth(URL_ARGS, int level);





/* Serial ------------------------------------------------------------*/

#if defined(CK_ARG) && defined(PTHREAD_CANCELED)
struct serial;
void serial_init(CK_ARG, struct serial **ssc, void *(*func)(void *), void *priv);
void serial_shared(CK_ARG, struct serial *sc);
int serial_fd(const struct serial *sc);
void serial_reconf(CK_ARG, struct serial *sc);
int serial_params(CK_ARG, struct serial *sc, const char **port, char **argv);
void serial_destroy(CK_ARG, struct serial **ssc);
void serial_write(const struct serial *sc, const char *str);
int serial_read(const struct serial *sc, void *dst, int cmin, int cmax, int tmo);
int serial_stty(CK_ARG, struct serial *sc, const char *stty);
void serial_reopen(CK_ARG, struct serial *sc);
void serial_lock(struct serial *sc);
void serial_unlock(struct serial *sc);
#endif

/* Instance variable conveniences ------------------------------------*/

#define RealVar(structname, magicno, name, member, defval, format, action) \
int									\
cfg_##structname##_##name (CK_ARG, double *name)			\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	if (ck_RealVar(CK_PASS, name, &sc->member, defval, format)) {	\
		action;							\
	}								\
	return (0);							\
}

#define RealVar2(structname, magicno, name, member, defval, format, action) \
int									\
cfg_##structname##_##name (CK_ARG, const char **name)			\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	if (ck_RealVar2(CK_PASS, name, &sc->member, &sc->member##_str,	\
	    defval, format)) {						\
		action;							\
	}								\
	return (0);							\
}

#define UintVar(structname, magicno, name, member, defval, action)	\
int									\
cfg_##structname##_##name (CK_ARG, unsigned *name)			\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	if (ck_UintVar(CK_PASS, name, &sc->member, defval)){		\
		action;							\
	}								\
	return (0);							\
}

#define WordVar(structname, magicno, name, member, action)		\
int									\
cfg_##structname##_##name (CK_ARG, const char **name)			\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	if (ck_WordVar(CK_PASS, name, &sc->member)) {			\
		action;							\
	}								\
	return (0);							\
}

#define GROUP(structname, magicno)					\
struct ck_instance *							\
cfg_##structname##_new(CK_ARG, unsigned grp)				\
{									\
	struct structname *sc;						\
	struct group *gp;						\
									\
	gp = CreateGroup(NULL, #structname, grp);			\
	if (gp == NULL) {						\
		ck_printf(CK_PASS,					\
		    "Already a group numbered %u\n", grp);		\
		return (NULL);						\
	}								\
	ALLOC_OBJ(sc, magicno);						\
	sc->cki.obj = sc;						\
	sc->gp = gp;							\
	AZ(pthread_mutex_init(&sc->lock, NULL));			\
	structname##_init(CK_PASS, sc);					\
	return (&sc->cki);						\
}									\
 									\
int 									\
cfg_##structname##_destroy(CK_ARG)					\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	structname##_deinit(CK_PASS, sc);				\
	DestroyGroup(sc->gp);						\
	AZ(pthread_mutex_destroy(&sc->lock));				\
	FREE_OBJ(sc);							\
	return (0);							\
}									\
									\
int									\
cfg_##structname##_point(CK_ARG, const char **point, char **argv)		\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	return (ConfigPoint(CK_PASS, sc->gp, point, argv));		\
}

#define SerialFunc(structname, magicno, name)				\
int									\
cfg_##structname##_##name (CK_ARG, const char **port, char **argv)	\
{									\
	struct structname *sc;						\
									\
	CAST_OBJ_NOTNULL(sc, cfa->priv, magicno);			\
	return (serial_params(CK_PASS, sc->name, port, argv));		\
}

/* Time convenience --------------------------------------------------*/

struct timespec Now(void);
struct timespec Utc(void);
double delta_t(const struct timespec *t1, const struct timespec *t2);

/* Pthread convenience -----------------------------------------------*/

#ifdef assert
#define AZ(foo)				\
	do {				\
		assert((foo) == 0);	\
	} while (0)
#endif

#if defined(PTHREAD_CANCELED)

void pthread_cancelmutex_lock(pthread_mutex_t *mp);
void pthread_cancelmutex_unlock(pthread_mutex_t *mp);

#endif

/* Malloc convenience ------------------------------------------------*/

#define ALLOC(ptr, size) do {			\
	(*(ptr)) = calloc(size, 1);		\
	assert(*(ptr) != NULL);			\
} while (0)

#define FREE(ptr) do {				\
	free(*(ptr));				\
	(*(ptr)) = NULL;			\
} while (0)
