/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: measured_int.h,v 1.71 2008/04/29 19:45:14 phk Exp $
 *
 */

#include <pthread.h>
#include <sys/queue.h>

struct sbuf;

#define	DEFAULT_POINT_TIMEOUT	120.0

TAILQ_HEAD(sitehead, site);
TAILQ_HEAD(grouphead, group);
TAILQ_HEAD(pointhead, point);

struct site {
	unsigned		magic;
#define SITE_MAGIC 0x3845696
	unsigned		instance;
	TAILQ_ENTRY(site)	list;
	struct grouphead	groups;
};

struct group {
	unsigned		magic;
#define GROUP_MAGIC 0x1103248
	unsigned		instance;
	TAILQ_ENTRY(group)	list;
	struct pointhead	points;

	char			*name;
	struct site		*site;
};

struct point {
	unsigned		magic;
#define POINT_MAGIC 0x94847322
	unsigned		instance;
	TAILQ_ENTRY(point)	list;
	struct group		*group;

	enum point_type		type;

	/* control stuff */
	control_t		*control;
	void			*control_priv1;
	void			*control_priv2;

	/* Status */
	double			raw;
	double			value;
	struct timespec		measured;

	/* Limit */
	struct timespec		limtimer;
	enum limit_e		prevlim;
	enum limit_e		curlim;

	/* Properties */
	char			*label;	
	char			*format;
	char			*unit;	
	char			*hysteresis_str;
	double			hysteresis;	
	double			limitdelay;
	char			*offset_str;	
	double			offset;	
	char			*scale_str;
	double			scale;
	char			*limits_str[7];	/* L_LOW ... L_CHANGED */
	double			limits[7];	/* L_LOW ... L_CHANGED */
	unsigned		trap;

	struct state		*state_p;
	unsigned		state;
	enum statepri_e		state_pri;

	/* Default Properties */
	char			*d_label;
	char			*d_format;
	char			*d_unit;
	double			d_timeout;
};

extern struct sitehead all_sites;
extern struct site *site_zero;		/* "local site" */

extern pthread_mutex_t	glbmtx;
#define Lock()		AZ(pthread_mutex_lock(&glbmtx))
#define Unlock()	AZ(pthread_mutex_unlock(&glbmtx))

void TopoHold(void);
void TopoRel(void);

struct event_arg {
	event_t 	e;
	void 		*obj;
	unsigned 	instance;
	const char	*name;
	char 		**string_cache;
	char 		*free_me;
};

typedef void event_handler_t(void *p, struct event_arg *e);

struct event_subscription;
struct event_subscription *SubscribeEvents(event_handler_t *func, void *priv);
int StartEvents(struct event_subscription *, const char *);
void UnsubscribeEvents(struct event_subscription *);
void PostEvent(event_t e, void *obj, unsigned instance);
const char *Event_String(struct event_arg *e);

/* state.c */
void StatePoint(struct point *pp);
void StateLimit(struct point *pp, enum limit_e nlimit);
void StateMaster(struct sbuf *sb, void *ptr);
int StateDump(void **p);

/* tcp.c */
typedef void tcp_handler_t(int sock);
void CheckSock(int sock, const char *func, int line);

struct tcp_listener *TcpListenTo(const char *port, tcp_handler_t *func, const char *name);
void TcpUnlistenTo(struct tcp_listener *p);

typedef void record_f(unsigned point, unsigned len, const void *ptr);

extern volatile record_f *record_function;

/* Init functions ----------------------------------------------------*/

extern struct ck_tree *cft;

void InitHttpd(void);
void InitUrlPages(void);
void InitPoint(void);
void InitTopo(void);
void InitUrlFavicon(void);
void InitUrlPattern(void);
void InitUrlAlarm(void);
void InitHttpdAuth(void);
void InitMaster(void);
void InitLog(void);
