#!/usr/local/bin/wish8.4
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42):
# <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
# ----------------------------------------------------------------------------
# $Id: measured_lib.tcl,v 1.1 2008/05/28 18:17:02 phk Exp $

set cvs {$Id: measured_lib.tcl,v 1.1 2008/05/28 18:17:02 phk Exp $}
set cvs_id($cvs) ""

proc measured_retry {var args} {
	upvar #0 $var m

	catch {close $m(s)}
	set m(s) ""
	puts stderr "Measured: Retry \"$args\""
	if {[catch "$m(los) $var" a]} {
		puts stderr "Measured: $var LOS: $a"
	}
	set m(retry_id) [after $m(retry) "measured_open $var"]
	after cancel $m(timer)
	unset m(timer)
}

proc measured_event {d var} {
	upvar #0 $var m

	after cancel $m(timer)
	unset m(timer)
	set m(timer) [after 60000 "measured_retry $var timeout"]
	if {[eof $m(s)]} {
		measured_retry $var "gets: $var"
		return;
	}
	set b ""

	# If the connect succeeds normaly, we get a write event
	# all other states, and cases need to read.
	if {$d == "W"} {
		fileevent $m(s) writable ""
	} else {
		if {[catch {gets $m(s) b} a]} {
			measured_retry $var "gets: $var"
			return;
		}
	}
	if {[string is control -strict -failindex a $b]} {
		measured_retry $var "control char"
		return
	}
	if {$b != "" && [regexp $b {[{}$;\[\]]}]} {
		measured_retry $var "bad char <$b>"
		return
	}
	switch $m(state) {
		connect	{
			if {[catch {puts $m(s) "GET /master HTTP/1.1\r\n\r"} a]} {
				measured_retry $var "puts($var)"
				return
			}
			flush $m(s)
			set m(state) http_resp
		}
		http_resp {
			if {$b != "HTTP/1.1 200 OK"} {
				measured_retry $var "http($var): $b"
				return
			}
			set m(state) http_hdr
		}
		http_hdr {
			if {$b != ""} {
				return
			}
			set m(state) online
			if {[info exists m(pattern)]} {
				puts $m(s) "PATTERN $m(pattern)"
			}
			puts $m(s) "EVENTS $m(events)"
			flush $m(s)
		}
		online {
			set m(data) $b
			if {[catch "$m(callback) $var" a]} {
				measured_retry $var "CALLBACK: $a"
			}
		}
		*	{
			measured_retry $var "state($var): $m(state"
			exit 0
			
		}
	}
}

proc measured_open {var} {
	upvar #0 $var m

	set m(state) offline
	puts "measured_open <$var>"
	catch {after cancel m(retry_id)}
	if {[catch "socket -async $m(host) $m(port)" m(s)]} {
		puts "Measured connect failed: $m(s)"
		set m(s) -1
		after $m(retry) "measured_open $var"
		return;
	}
	set m(state) connect
	fconfigure $m(s) -blocking false -buffering line
	fileevent $m(s) readable "measured_event R $var"
	fileevent $m(s) writable "measured_event W $var"
	set m(timer) [after 60000 "measured_retry $var timeout"]
}

proc measured_isready {var} {
	upvar #0 $var m

	if {$m(state) != "online"} {
		return 0
	} 
	return 1
}

proc measured_pulse {var pt val dur val2} {
	upvar #0 $var m
	puts "PULSE point $pt value $val"
	if {![measured_isready $var]} {
		return 0
	}
	puts $m(s) "CONTROL $pt $val"
	after $dur "puts $m(s) \"CONTROL $pt $val2\""
	return 1
}

proc measured_control {var pt val} {
	upvar #0 $var m
	puts "CONTROL point $pt value $val"
	if {![measured_isready $var]} {
		return 0
	}
	puts $m(s) "CONTROL $pt $val"
	return 1
}

# set measured(host) $meteo_host
#set measured(port) 2040
#set measured(pattern) 0
#set measured(events) ""
#set measured(retry) 3000
# set measured(callback) readf

#measured_open measured
